/*
 * File nios_cover-ucos.c
 *
 * Authors Philip Duval (DESY), Steve Herb (DESY), Marina Nikolova (EMBL), 
 *    Karol Bartkiewicz (DESY), Marcus Walla (DESY), Jan Szczesny (DESY),
 *    Arthur Aghababyan (DESY), Stefan Weisse (DESY), Tobias Tempel (DESY)
 *
 * Copyright (c) Deutsches Elektronen-Synchrotron DESY 1992-2023
 *
 * The TINE .c codeset made available by open source license ultimately
 * stems from the original ISOLDE control system ((c)CERN, released under
 * a permissive CERN licence akin to the later well known MIT licence)
 * in use at CERN in 1992.  Although none of the current software is in
 * anyway a redistribution of software from the ISOLDE system of that
 * era (apart from various error/status code definitions), it does in
 * fact inherit several conceptual ideas.  We therefore owe a substantial
 * debt to CERN for its inception, and would be remiss if we did not express
 * our gratitude in the TINE open source headers.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "nios.h"
#include "stdio.h"
#include <time.h>
#include <sys/time.h>

#include <ucosii/os_cfg.h>
#include <ucosii/os_cpu.h>


/* we aren't doing any file IO so dummy these routines for now ... */
int _open(const char *filename,int oflag,int pmode)
{
  return -1; /* return failure */
}
int _unlink(const char *filename)
{
  return 0;
}
int _link(const char *filename)
{
  return -1;
}
int nr_fprintf(FILE *fp, const char *fmt, ... )
{
	return 0; /* return failure */
}
/* These two functions below are important ! */

long time(long *lptime)
{
  static INT32U firsttime;
  INT32U currtime;
  static int done=0;

  if (done == 0)
  {
   done = 1;
   firsttime = OSTimeGet();
  }
  
  currtime = OSTimeGet();
  if (firsttime > currtime) firsttime = currtime;

  if (lptime != NULL) *lptime = (currtime-firsttime) / OS_TICKS_PER_SEC;

  return( (currtime-firsttime) / OS_TICKS_PER_SEC );
}

// get current usec fraction for gettimeofday
long timeusec(long *lptime)
{
  static INT32U firsttime;
  INT32U currtime;
  static int done=0;

  if (done == 0)
  {
   done = 1;
   firsttime = OSTimeGet();
  }
  
  currtime = OSTimeGet();
  if (firsttime > currtime) firsttime = currtime;

  if (lptime != NULL) *lptime = (((currtime-firsttime) % OS_TICKS_PER_SEC)*1000000) / OS_TICKS_PER_SEC;

  return( (((currtime-firsttime) % OS_TICKS_PER_SEC)*1000000) / OS_TICKS_PER_SEC );
}


struct timeval *_gettimeofday(struct timeval *tv,struct timezone *tz)
{
 tv->tv_sec = time(NULL);
 tv->tv_usec = timeusec(NULL);

 return( tv );
}

/* running count since last boot */
/* later either use:
   1. TINE timestamp synchronization to apply an adjustment or
   2. port the rdate daemon from VxWorks to NIOS
*/

/*
  static int tsec = 0;  // adjusted offset for potential wrapping of tmsec 
  static int tmsec = 0; // result of most recent call 
  int nt = nr_timer_milliseconds(); // this call 
  if (nt < tmsec) tsec += tmsec/1000; // it has wrapped 
  tmsec = nt;
  if (tv != NULL)
  {
    tv->tv_sec = tsec + tmsec/1000;
    tv->tv_usec = (tmsec % 1000) * 1000;
  }
  return tv;
*/

